package com.brilliance.isc.swift.objmtconvert.model.mt2xx;

import com.brilliance.isc.swift.objmtconvert.annotation.FieldDescription;
import com.brilliance.isc.swift.objmtconvert.annotation.Status;
import com.brilliance.isc.swift.objmtconvert.annotation.TagName;
import com.brilliance.isc.swift.objmtconvert.annotation.TagRegex;
import com.brilliance.isc.swift.objmtconvert.model.AbsMTCreator;
import com.brilliance.isc.swift.objmtconvert.model.AbsTagCreate;
import com.brilliance.isc.swift.objmtconvert.model.commontag.ActnoBicAdrTagXXX;
import com.brilliance.isc.swift.objmtconvert.model.commontag.DcDateCurAmtTagXXX;
import com.brilliance.isc.swift.objmtconvert.model.mt2xx.tag.tag290.*;
import com.brilliance.isc.swift.objmtconvert.result.MtResult;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description：
 * @Author：夏猛
 * @CreateTime：2023/2/28 10:04
 **/
public class MT290Creator extends AbsMTCreator {

    //20
    @Status("M")
    @TagName("20")
    @FieldDescription("Transaction Reference Number")
    @TagRegex("16x")
    private String transactionReferenceNumber;

    //21
    @Status("M")
    @TagName("21")
    @FieldDescription("Related Reference")
    @TagRegex("16x")
    private String relatedReference;

    //25
    @Status("M")
    @TagName("25")
    @FieldDescription("Account Identification")
    @TagRegex("35x")
    private String accountIdentification;

    //32a
    @Status("M")
    @TagName("32D")
    @FieldDescription("Ordering Institution")
    @TagRegex("6!n3!a15d")
    private DcDateCurAmtTagXXX valueDateCurrencyCodeAmount;

    //52a
    @Status("O")
    @TagName({"52A","52D"})
    @FieldDescription("Ordering Institution")
    @TagRegex({"[/1!a][/34x]<crlf>4!a2!a2!c[3!c]","[/1!a][/34x]<crlf>4*35x"})
    private ActnoBicAdrTagXXX orderingInstitution;

    //71B
    @Status("M")
    @TagName("71B")
    @FieldDescription("Details of Charges")
    @TagRegex("6*35x")
    private String detailsofCharges;

    //72
    @Status("O")
    @TagName("72")
    @FieldDescription("Sender to Receiver Information")
    @TagRegex("6*35x")
    private String sendertoReceiverInformation;

    public String getTransactionReferenceNumber() {
        return transactionReferenceNumber;
    }

    public void setTransactionReferenceNumber(String transactionReferenceNumber) {
        this.transactionReferenceNumber = transactionReferenceNumber;
    }

    public String getRelatedReference() {
        return relatedReference;
    }

    public void setRelatedReference(String relatedReference) {
        this.relatedReference = relatedReference;
    }

    public String getAccountIdentification() {
        return accountIdentification;
    }

    public void setAccountIdentification(String accountIdentification) {
        this.accountIdentification = accountIdentification;
    }

    public DcDateCurAmtTagXXX getValueDateCurrencyCodeAmount() {
        return valueDateCurrencyCodeAmount;
    }

    public void setValueDateCurrencyCodeAmount(DcDateCurAmtTagXXX valueDateCurrencyCodeAmount) {
        this.valueDateCurrencyCodeAmount = valueDateCurrencyCodeAmount;
    }

    public ActnoBicAdrTagXXX getOrderingInstitution() {
        return orderingInstitution;
    }

    public void setOrderingInstitution(ActnoBicAdrTagXXX orderingInstitution) {
        this.orderingInstitution = orderingInstitution;
    }

    public String getDetailsofCharges() {
        return detailsofCharges;
    }

    public void setDetailsofCharges(String detailsofCharges) {
        this.detailsofCharges = detailsofCharges;
    }

    public String getSendertoReceiverInformation() {
        return sendertoReceiverInformation;
    }

    public void setSendertoReceiverInformation(String sendertoReceiverInformation) {
        this.sendertoReceiverInformation = sendertoReceiverInformation;
    }

    public MT290Creator(String msgType)  {
        super(msgType);
    }

    public MT290Creator() {
        super();
    }
    /**
     * 需要编写组装报文逻辑，调用set方法即可
     *
     * @return
     */
    public void pack() {
        MtResult mtResult = new MtResult();
        List<AbsTagCreate> tagCreateList=new ArrayList<>();
        try {
            tagCreateList.add(new Tag20Creator(this));
            tagCreateList.add(new Tag21Creator(this));
            tagCreateList.add(new Tag25Creator(this));
            tagCreateList.add(new Tag32aCreator(this));
            tagCreateList.add(new Tag52aCreator(this));
            tagCreateList.add(new Tag71BCreator(this));
            tagCreateList.add(new Tag72Creator(this));

            dealTags(tagCreateList);

            mtResult.setMessage(abstractMT.message());
        } catch (Exception e) {
            e.printStackTrace();
            mtResult.setSuccess(false);
            mtResult.addDescription(e.getLocalizedMessage());
        }

        //  return mtResult;
    }

    public MtResult unpack() {
        return null;
    }

}