package com.brilliance.isc.swift.objmtconvert.model.mt7xx;

import com.brilliance.isc.swift.objmtconvert.annotation.FieldDescription;
import com.brilliance.isc.swift.objmtconvert.annotation.Status;
import com.brilliance.isc.swift.objmtconvert.annotation.TagName;
import com.brilliance.isc.swift.objmtconvert.annotation.TagRegex;
import com.brilliance.isc.swift.objmtconvert.model.AbsMTCreator;
import com.brilliance.isc.swift.objmtconvert.model.AbsTagCreate;
import com.brilliance.isc.swift.objmtconvert.model.commontag.ActnoBicAdrTagXXX;
import com.brilliance.isc.swift.objmtconvert.model.commontag.DcDateCurAmtTagXXX;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag20Creator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag21Creator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag23XCreator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag52aCreator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag72ZCreator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag30Creator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag32BCreator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag77BCreator;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag786.Tag77JCreator;
import com.brilliance.isc.swift.objmtconvert.result.MtResult;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wuzhe
 * @date 2023/2/27
 */
@Data
public class MT786Creator extends AbsMTCreator {

    @Status("M")
    @TagName("20")
    @FieldDescription("Undertaking Number")
    @TagRegex("16x")
    //20
    private String undertakingNumber;
    @Status("O")
    @TagName("21")
    @FieldDescription("Beneficiary Reference Number")
    @TagRegex("16x")
    //21
    private String beneficiaryReferenceNumber;
    @Status("M")
    @TagName({"52A","52D"})
    @FieldDescription("Issuer")
    @TagRegex({"[/1!a][/34x]<crlf>4!a2!a2!c[3!c]","[/1!a][/34x]<crlf>4*35x"})
    //52a
    private ActnoBicAdrTagXXX issuer;
    @Status("M")
    @TagName("30")
    @FieldDescription("Demand Submission Date")
    @TagRegex("6!n")
    //30
    private String demandSubmissionDate;
    @Status("M")
    @TagName("32B")
    @FieldDescription("Demand Amount")
    @TagRegex("3!a15d")
    //32B
    private DcDateCurAmtTagXXX demandAmount;
    @Status("M")
    @TagName("77J")
    @FieldDescription("Reason for Refusal")
    @TagRegex("70*50z")
    //77J
    private ActnoBicAdrTagXXX reasonforRefusal;
    @Status("O")
    @TagName("77B")
    @FieldDescription("Disposal of Documents")
    @TagRegex("3*35x")
    //77B
    private String disposalofDocuments;
    @Status("O")
    @TagName("72Z")
    @FieldDescription("Sender to Receiver Information")
    @TagRegex("6*35z")
    //72Z
    private ActnoBicAdrTagXXX sendertoReceiverInformation;
    @Status("O")
    @TagName("23X")
    @FieldDescription("File Identification")
    @TagRegex("4!c/65x")
    //23X
    private String fileIdentification;



    public MT786Creator(String msgType)  {
        super(msgType);
    }

    public MT786Creator() {
        super("786");
    }

    /**
     * 需要编写组装报文逻辑，调用set方法即可
     *
     * @return
     */
    public void pack() {
       // MtResult mtResult = new MtResult();
        List<AbsTagCreate> tagCreateList = new ArrayList<>();
        try {
            tagCreateList.add(new Tag20Creator(this));
            tagCreateList.add(new Tag21Creator(this));
            tagCreateList.add(new Tag52aCreator(this));
            tagCreateList.add(new Tag30Creator(this));
            tagCreateList.add(new Tag32BCreator(this));
            tagCreateList.add(new Tag77JCreator(this));
            tagCreateList.add(new Tag77BCreator(this));
            tagCreateList.add(new Tag72ZCreator(this));
            tagCreateList.add(new Tag23XCreator(this));
            dealTags(tagCreateList);

            mtResult.setMessage(abstractMT.message());
        } catch (Exception e) {
            e.printStackTrace();
            mtResult.setSuccess(false);
            mtResult.addDescription(e.getLocalizedMessage());
        }

        //return mtResult;

    }

    public MtResult unpack() {
        return null;
    }

}

