package com.brilliance.isc.swift.objmtconvert.model.mt7xx;

import com.brilliance.isc.swift.objmtconvert.annotation.FieldDescription;
import com.brilliance.isc.swift.objmtconvert.annotation.Status;
import com.brilliance.isc.swift.objmtconvert.annotation.TagName;
import com.brilliance.isc.swift.objmtconvert.annotation.TagRegex;
import com.brilliance.isc.swift.objmtconvert.model.AbsMTCreator;
import com.brilliance.isc.swift.objmtconvert.model.AbsTagCreate;
import com.brilliance.isc.swift.objmtconvert.model.commontag.ActnoBicAdrTagXXX;
import com.brilliance.isc.swift.objmtconvert.model.mt7xx.tag.tag787.*;
import com.brilliance.isc.swift.objmtconvert.result.MtResult;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.ArrayList;
import java.util.List;

@EqualsAndHashCode(callSuper = true)
@Data
public class MT787Creator extends AbsMTCreator {

    @Status("M")
    @TagName("20")
    @FieldDescription("Transaction Reference Number")
    @TagRegex("16x")
    //20
    private String transactionReferenceNumber;

    @Status("M")
    @TagName("21")
    @FieldDescription("Related Reference")
    @TagRegex("16x")
    //21
    private String relatedReference;

    @Status("M")
    @TagName({"52A","52D"})
    @FieldDescription("Issuer")
    @TagRegex({"[/1!a][/34x]<crlf>4!a2!a2!c[3!c]","[/1!a][/34x]<crlf>4*35x"})
    //52a
    private ActnoBicAdrTagXXX issuer52a;

    @Status("M")
    @TagName({"26E"})
    @FieldDescription("Number of Amendment")
    @TagRegex({"3n"})
    //52a
    private String numberofAmendment;

    @Status("M")
    @TagName({"23R"})
    @FieldDescription("Amendment Status")
    @TagRegex({"4!c<crlf>[6*35x]"})
    //23R
    private String amendmentStatus;

    @Status("O")
    @TagName({"72Z"})
    @FieldDescription("Sender to Receiver Information")
    @TagRegex({"6*35z"})
    //72Z
    private String sendertoReceiverInformation;

    @Status("O")
    @TagName({"23X"})
    @FieldDescription("File Identification")
    @TagRegex({"4!c/65x"})
    //72Z
    private String fileIdentification;

    public MT787Creator(String msgType)  {
        super(msgType);
    }

    public MT787Creator() {
        super("787");
    }


    public void pack() {
        //MtResult mtResult = new MtResult();
        List<AbsTagCreate> tagCreateList = new ArrayList<>();
        try {
            tagCreateList.add(new Tag20Creator(this));
            tagCreateList.add(new Tag21Creator(this));
            tagCreateList.add(new Tag52aCreator(this));
            tagCreateList.add(new Tag26ECreator(this));
            tagCreateList.add(new Tag23RCreator(this));
            tagCreateList.add(new Tag72ZCreator(this));
            tagCreateList.add(new Tag23XCreator(this));
            dealTags(tagCreateList);

            mtResult.setMessage(abstractMT.message());
        } catch (Exception e) {
            e.printStackTrace();
            mtResult.setSuccess(false);
            mtResult.addDescription(e.getLocalizedMessage());
        }

        // return mtResult;

    }

    public MtResult unpack() {
        return null;
    }
}
